# Copyright (C) 2009, 2010 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest import TestCase
import datetime

from apache_openid import get_action_path, protect, protect_noactions
from apache_openid.handlers.openid.handler import OpenIDLoginHandler
from apache_openid.handlers.openidteams.handler import OpenIDTeamsLoginHandler
from apache_openid.request import Request
from apache_openid.response import Response
from apache_openid.utils import DONE, HTTP_FORBIDDEN, OK, SERVER_RETURN
from apache_openid.utils.mock import ApacheMockRequest, Options, Session


class HandlerTestCase(TestCase):
    overrides = {
        'action-path': "/openid/",
    }

    def setUp(self):
        self.apache_request = ApacheMockRequest(overrides=self.overrides)
        self.options = self.apache_request.get_options()
        self.action_path = get_action_path(self.options, self.apache_request)
        self.session = Session()
        self.request = Request(
            self.apache_request, self.action_path, self.session)
        self.response = Response(
            self.request, self.action_path, self.session)
        self.handler = OpenIDLoginHandler(
            self.request, self.response, self.options, None, None)

    def assertRaisesMessage(self, exc, call, msg):
        try:
            result = call()
        except Exception, e:
            self.assertEqual(isinstance(e, exc), True)
            self.assertEqual(e.args[0], msg)
        else:
            self.fail("No exception raised. Got a result instead: %s" % result)


class OpenIDLoginHandlerTest(HandlerTestCase):

    def test_authenticator(self):
        self.assertEqual(getattr(self.handler, '_auth', None), None)
        authenticator = self.handler.authenticator
        self.assertNotEqual(getattr(self.handler, '_auth', None), None)

    def test_consumer(self):
        self.assertEqual(getattr(self.handler, '_consumer', None), None)
        consumer = self.handler.consumer
        self.assertNotEqual(getattr(self.handler, '_consumer', None), None)

    def test_known_actions(self):
        self.assertEqual(
            self.handler.known_actions,
            ['+logout', '+login', '+return'])

    def test_protect(self):
        self.assertRaisesMessage(SERVER_RETURN, self.handler.protect, DONE)

    def test_protect_favicon(self):
        self.request.request.subprocess_env['REQUEST_URI'] = '/favicon.ico'
        self.assertEqual(self.handler.protect(), OK)

    def test_protect_post(self):
        self.handler.request.request.method = 'POST'
        self.assertRaisesMessage(
            SERVER_RETURN, self.handler.protect, HTTP_FORBIDDEN)

    def test_protect_cookied_user(self):
        self.handler.request.cookied_user = []
        self.assertRaisesMessage(
            SERVER_RETURN, self.handler.protect, DONE)

class TeamsLoginHandlerTest(HandlerTestCase):
    overrides = {
        'authorized-teams': "canonical-isd-hackers bzr-core",
        'action-path': "/openid/",
    }

    def setUp(self):
        super(TeamsLoginHandlerTest, self).setUp()
        self.handler = OpenIDTeamsLoginHandler(
            self.request, self.response, self.options, self.session, None)

    def test_teams_header_is_set(self):
        my_team = 'some-team'
        expiry = datetime.datetime.now() + datetime.timedelta(hours=1)
        self.session['cookied_teams'] = {my_team: expiry}
        self.request.request.subprocess_env['REQUEST_URI'] = '/favicon.ico'
        self.handler.protect()
        self.assertEqual(my_team, self.request.headers_in['X-OpenID-teams'])

    def test_teams_header_is_reset(self):
        self.request.headers_in['X-OpenID-teams'] = 'some-super-team'
        self.request.request.subprocess_env['REQUEST_URI'] = '/favicon.ico'
        self.handler.protect()
        self.assertEqual('', self.request.headers_in['X-OpenID-teams'])

class TopLevelHandlersTest(TestCase):
    def setUp(self):
        self.apache_request = ApacheMockRequest()

    def test_protect_pass(self):
        self.apache_request.subprocess_env['REQUEST_URI'] = '/favicon.ico'
        self.assertEqual(OK, protect(self.apache_request))
        

    def test_protect_actions(self):
        self.apache_request.subprocess_env['REQUEST_URI'] = '/+login'
        self.assertEqual(OK, protect(self.apache_request))
        self.assertEqual(1, len(self.apache_request.handlers))
        self.assertEqual('PythonHandler', self.apache_request.handlers[0][0])

    def test_protect_noactions(self):
        self.apache_request.subprocess_env['REQUEST_URI'] = '/favicon.ico'
        self.assertEqual(OK, protect_noactions(self.apache_request))
        self.assertEqual(0, len(self.apache_request.handlers))
